package com.persagy.server.syn.services.dict

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dict.DClassDef
import com.persagy.server.datacenter.models.entities.dictnew.*
import com.persagy.server.syn.models.api.DictTypeApi
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import org.slf4j.LoggerFactory

/**
 * 字典类型
 *
 * @author 张维新
 */
object DClassDefService : SObjectService<DClassDef>(SMybatisDao(DClassDef::class.java)) {

    /** 日志 */
    val logger = LoggerFactory.getLogger(DClassDefService::class.java)
    /** 集团服务 */
    private val groupService = SBaseService(SMybatisDao(Group::class.java))
    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))
    /** 字典平台对象类型 */
    private val defClassService = SBaseService(SMybatisDao(DefClass::class.java))
    /** 详细信息点 */
    private val defFuncIdService = SBaseService(SMybatisDao(DefFuncId::class.java))
    /** 专业 */
    private val dictMajorService = SBaseService(SMybatisDao(DictMajor::class.java))
    /** 方案 */
    private val dictSchemeService = SBaseService(SMybatisDao(DictScheme::class.java))
    /** 方案内对象类型 */
    private val dictSchemeCategotyService = SBaseService(SMybatisDao(DictSchemeCategoty::class.java))
    /** 方案内对象包含的信息点 */
    private val dictSchemeCategotyInfoService = SBaseService(SMybatisDao(DictSchemeCategotyInfo::class.java))
    /** 方案内对象包含的信息点 */
    private val dictSchemeAndProjectService = SBaseService(SMybatisDao(DictSchemeAndProject::class.java))

    /**
     * 创建字典类型
     */
    fun create() {
        DictTypeApi.dictTypeHttp()

    } //

    /**
     * 查询集团信息
     */
    fun groupQuerys(): SBaseResponse {
        val sBaseResponse = SBaseResponse()
        val groupQueryList = DictTypeApi.groupQuery()
        logger.debug("集团结果:${groupQueryList.toJson()}")
        if (groupQueryList.size > 0) {
            groupService.deleteAll()
            for (groupQuery in groupQueryList) {
                val group = Group()
                group.id = groupQuery.code
                group.name = groupQuery.name
                groupService.insert(group)
            }
        } else {
            sBaseResponse.result = SResponseType.failure
            return sBaseResponse
        }
        sBaseResponse.result = SResponseType.success
        return sBaseResponse
    } // Fun groupQuerys()

    /**
     * 集团下的项目
     */
    fun dictProject(): SBaseResponse {
        val sBaseResponse = SBaseResponse()
        try {
            val selectAllList = groupService.selectAll().exec()
            logger.debug("集团信息:${selectAllList.toJson()}")
            if (!selectAllList.isNullOrEmpty()) {
                for (group in selectAllList) {

                    val dictProjectQueryList = DictTypeApi.dictProjectQuery(group.id!!)
                    if (dictProjectQueryList.size > 0) {
                        logger.debug("项目结果:${dictProjectQueryList.toJson()}")
                        dictProjectService.delete(SFilter.eq("groupId",group.id!!))
                        for (dictProjectQuery in dictProjectQueryList) {
                            val dictProject = DictProject()
                            dictProject.id = dictProjectQuery.id
                            dictProject.name = dictProjectQuery.name
                            dictProject.groupId = group.id
                            dictProjectService.insert(dictProject)
                        }
                    }
                }
                sBaseResponse.result = SResponseType.success
                return sBaseResponse
            } else {
                sBaseResponse.result = SResponseType.failure
                sBaseResponse.message = "请先获取集团信息"
                return sBaseResponse
            }
        } catch (e: Exception) {
            e.printStackTrace()
            sBaseResponse.result = SResponseType.failure
            return sBaseResponse
        }
    } // Fun dictProject()

    /**
     * 平台字典 对象类型
     */
    fun dictPlatform(): SBaseResponse {
        val sBaseResponse = SBaseResponse()
        val dictPlatformList = DictTypeApi.dictPlatform()
        if (dictPlatformList.size>0){
            defClassService.deleteAll()
            for (dictPlatform in dictPlatformList){
                val defClass = DefClass()
                defClass.aliasCode = dictPlatform.aliasCode
                defClass.aliasName = dictPlatform.aliasName
                defClass.code = dictPlatform.code
                defClass.equipmentCode = dictPlatform.equipmentCode
                defClass.groupId = if (dictPlatform.groupId.isNullOrEmpty())  "0" else dictPlatform.groupId
                defClass.majorCode = dictPlatform.majorCode
                defClass.name = dictPlatform.name
                defClass.parentCode = dictPlatform.parentCode
                defClass.projectId = if (dictPlatform.projectId.isNullOrEmpty())  "0" else dictPlatform.projectId
                defClass.systemCode = dictPlatform.systemCode
                defClass.type = "common"
                defClassService.replace(defClass)
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return sBaseResponse
    } // fun dictPlatform()

    /**
     * 字典对象的详细信息点
     */
    fun dictPlatformInfo(): SBaseResponse{
        val sBaseResponse = SBaseResponse()
        val defClassList = defClassService.select(SFilter.eq("type", "common")).exec()
        if (defClassList.size>0){
            defFuncIdService.delete(SFilter.eq("type", "common"))
            for (defClass in defClassList){
                val dictPlatformInfoList = DictTypeApi.dictPlatformInfo(defClass.code!!, defClass.type!!)
                if (!dictPlatformInfoList.isNullOrEmpty()){
                    for (dictPlatformInfo in dictPlatformInfoList){
                        val defFuncId = DefFuncId()
                        defFuncId.canSub = dictPlatformInfo.canSub
                        defFuncId.category = dictPlatformInfo.category
                        defFuncId.classCode = defClass.code
                        defFuncId.code = dictPlatformInfo.code
                        defFuncId.dataSource = dictPlatformInfo.dataSource
                        defFuncId.dataType = dictPlatformInfo.dataType
                        defFuncId.firstTag = dictPlatformInfo.firstTag
                        defFuncId.formater = dictPlatformInfo.formater
                        defFuncId.groupId = if (dictPlatformInfo.groupId.isNullOrEmpty()) "0" else dictPlatformInfo.groupId
                        defFuncId.idx = dictPlatformInfo.idx
                        defFuncId.inputMode = dictPlatformInfo.inputMode
                        defFuncId.isMultiple = dictPlatformInfo.isMultiple
                        defFuncId.isRegion = dictPlatformInfo.isRegion
                        defFuncId.name = dictPlatformInfo.name
                        defFuncId.note = dictPlatformInfo.note
                        defFuncId.priority = dictPlatformInfo.priority
                        defFuncId.projectId = if (dictPlatformInfo.projectId.isNullOrEmpty()) "0" else dictPlatformInfo.projectId
                        defFuncId.readonly = dictPlatformInfo.readonly
                        defFuncId.secondTag = dictPlatformInfo.secondTag
                        defFuncId.subFlag = dictPlatformInfo.subFlag
                        defFuncId.type = dictPlatformInfo.dataType
                        defFuncId.unit = dictPlatformInfo.unit
                        defFuncIdService.insert(defFuncId)
                    }
                }
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return sBaseResponse
    } // Fun dictPlatformInfo()

    /**
     * 专业
     */
    fun dictMajor(): SBaseResponse{
        val sBaseResponse = SBaseResponse()
        val dictMajorList = DictTypeApi.dictMajor("common")
        dictMajorService.delete(SFilter.eq("type","common"))
        if (dictMajorList.size>0){
            for (dictMajor in dictMajorList){

                val dictMajor1 = DictMajor()
                dictMajor1.aliasCode = dictMajor.aliasCode
                dictMajor1.aliasName = dictMajor.aliasName
                dictMajor1.code = dictMajor.code
                dictMajor1.groupId = dictMajor.groupId
                dictMajor1.id = dictMajor.id
                dictMajor1.name = dictMajor.name
                dictMajor1.note = dictMajor.note
                dictMajor1.type = dictMajor.type
                dictMajorService.insert(dictMajor1)
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return  sBaseResponse
    } // dictMajor()

    /**
     * 集团下的方案
     */
    fun dictGroupScheme(): SBaseResponse{
        val sBaseResponse = SBaseResponse()
        val groupList = groupService.selectAll().exec()
        if (groupList.size>0){

            for (group in groupList){
                dictSchemeService.delete(SFilter.eq("groupId",group.id!!))
                val dictGroupSchemeList = DictTypeApi.dictGroupScheme(group.id!!)
                if (dictGroupSchemeList.size>0){
                    for (dictGroupScheme in dictGroupSchemeList){
                        val dictScheme = DictScheme()
                        dictScheme.groupId = dictGroupScheme.groupId
                        dictScheme.id = dictGroupScheme.id
                        dictScheme.name = dictGroupScheme.name
                        dictScheme.note = dictGroupScheme.note
                        dictSchemeService.insert(dictScheme)
                    }
                }
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return  sBaseResponse
    } // Fun dictGroupScheme()

    /**
     * 方案下的对象类型
     */
    fun dictSchemeCategoty(): SBaseResponse{
        val sBaseResponse = SBaseResponse()
        val dictSchemeList = dictSchemeService.selectAll().exec()
        if (dictSchemeList.size>0){
            for (dictScheme in dictSchemeList){

                val dictSchemeCategotyList = DictTypeApi.dictSchemeCategoty(dictScheme.id!!)
                if (dictSchemeCategotyList.size>0){
                    dictSchemeCategotyService.delete(SFilter.eq("schemeId",dictScheme.id!!))
                    for (dictSchemeCategoty in dictSchemeCategotyList){
                        val dictSchemeCategoty1 = DictSchemeCategoty()
                        dictSchemeCategoty1.classCode = dictSchemeCategoty.classCode
                        dictSchemeCategoty1.id = dictSchemeCategoty.id
                        dictSchemeCategoty1.schemeId = dictSchemeCategoty.schemeId
                        dictSchemeCategotyService.replace(dictSchemeCategoty1)
                    }
                }
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return  sBaseResponse
    } // Fun dictSchemeCategoty()

    /**
     * 方案下的对象的信息点
     */
    fun dictSchemeCategotyInfo(): SBaseResponse{
        val sBaseResponse = SBaseResponse()
        val dictSchemeCategotyList = dictSchemeCategotyService.selectAll().exec()
        if (dictSchemeCategotyList.size>0){
            for (dictSchemeCategoty in dictSchemeCategotyList){
                val dictSchemeCategotyInfoList =
                    DictTypeApi.dictSchemeCategotyInfo(dictSchemeCategoty.schemeId!!, dictSchemeCategoty.classCode!!)
                dictSchemeCategotyInfoService.delete(SFilter.eq("schemeId",dictSchemeCategoty.schemeId!!),SFilter.eq("classCode",dictSchemeCategoty.classCode!!))
                if (dictSchemeCategotyInfoList.size>0){
                    for (dictSchemeCategotyInfo in dictSchemeCategotyInfoList){
                        val dictSchemeCategotyInfo1 = DictSchemeCategotyInfo()
                        dictSchemeCategotyInfo1.classCode = dictSchemeCategotyInfo.classCode
                        dictSchemeCategotyInfo1.id = dictSchemeCategotyInfo.id
                        dictSchemeCategotyInfo1.infoCode = dictSchemeCategotyInfo.infoCode
                        dictSchemeCategotyInfo1.schemeId = dictSchemeCategotyInfo.schemeId
                        dictSchemeCategotyInfoService.insert(dictSchemeCategotyInfo1)
                    }
                }
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return  sBaseResponse
    } // Fun dictSchemeCategotyInfo()

    /**
     * 查询项目和方案的关系
     */
    fun dictSchemeAndProject(): SBaseResponse{
        val sBaseResponse = SBaseResponse()
        val dictProjectList = dictProjectService.selectAll().exec()
        if (dictProjectList.size>0){
            dictSchemeAndProjectService.deleteAll()
            for (dictProject in dictProjectList){
                val dictSchemeAndProjectList = DictTypeApi.dictSchemeAndProject(dictProject.id!!)
                if (dictSchemeAndProjectList.size>0){
                    for (dictScheme in dictSchemeAndProjectList){
                        val dictSchemeAndProject = DictSchemeAndProject()
                        dictSchemeAndProject.id = dictScheme.id
                        dictSchemeAndProject.projectId = dictScheme.projectId
                        dictSchemeAndProject.schemeId = dictScheme.schemeId
                        dictSchemeAndProjectService.insert(dictSchemeAndProject)
                    }
                }
            }
            sBaseResponse.result = SResponseType.success
        } else {
            sBaseResponse.result = SResponseType.failure
        }
        return  sBaseResponse
    } // Fun dictSchemeAndProject()



} // Object DClassDefService
